package dev.fyloz.colorrecipesexplorer.config.security

import dev.fyloz.colorrecipesexplorer.dtos.account.GroupTokenDto
import dev.fyloz.colorrecipesexplorer.dtos.account.UserDetails
import dev.fyloz.colorrecipesexplorer.exception.NotFoundException
import dev.fyloz.colorrecipesexplorer.logic.account.GroupTokenLogic
import mu.KotlinLogging
import org.springframework.security.authentication.AuthenticationProvider
import org.springframework.security.authentication.BadCredentialsException
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken
import org.springframework.security.core.Authentication
import org.springframework.security.core.AuthenticationException
import java.util.*

class GroupTokenAuthenticationProvider(private val groupTokenLogic: GroupTokenLogic) : AuthenticationProvider {
    private val logger = KotlinLogging.logger {}

    override fun authenticate(authentication: Authentication): Authentication {
        val groupAuthenticationToken = authentication as GroupAuthenticationToken

        val groupToken = try {
            retrieveGroupToken(groupAuthenticationToken.id)
        } catch (e: AuthenticationException) {
            logger.debug(e.message)
            throw e
        }

        val userDetails =
            UserDetails(
                groupToken.id.toString(),
                groupToken.name,
                "",
                groupToken.group,
                groupToken.group.permissions,
                true
            )
        return UsernamePasswordAuthenticationToken(userDetails, null, userDetails.authorities)
    }

    override fun supports(authentication: Class<*>) =
        authentication.isAssignableFrom(GroupAuthenticationToken::class.java)

    private fun retrieveGroupToken(id: UUID): GroupTokenDto {
        val groupToken = try {
            groupTokenLogic.getById(id)
        } catch (_: NotFoundException) {
            throw BadCredentialsException("Failed to find group token with id '$id'")
        }

        if (groupTokenLogic.isDisabled(groupToken.id.toString())) {
            throw BadCredentialsException("Group token '${groupToken.id}' is disabled")
        }

        return groupToken
    }
}
